// Page pour les methodes communes e tous les tests

package pageObjects;

import org.junit.*;
import static org.junit.Assert.*;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.RandomStringUtils;
import org.openqa.selenium.*;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.chrome.ChromeOptions;
import org.openqa.selenium.firefox.FirefoxDriver;
import org.openqa.selenium.support.ui.ExpectedCondition;
import org.openqa.selenium.support.ui.WebDriverWait;

import utils.AbstractTest;

public class Page_Abstract extends AbstractTest {
	protected int timeOut = 5;	
	private static final Logger logger = LogManager.getLogger("Page_Abstract");
	//private static WebDriver driver;
	private boolean acceptNextAlert = true;
	private StringBuffer verificationErrors = new StringBuffer();

	public class Selecteur {
		public String nom;
		public By chemin;

		public Selecteur(String nom, By chemin){
			this.nom = nom;
			this.chemin = chemin;
		}

		public String getNom(){
			return nom;
		}

		public By getChemin(){
			return chemin;
		}
	}


	// Methodes techniques communes generees par SELENIUM IDE		
	public static boolean isElementPresent_old(By by) {
		try {
			driver.findElement(by);
			return true;
		} catch (NoSuchElementException e) {
			return false;
		}
	}

	public boolean isAlertPresent() {
		try {
			driver.switchTo().alert();
			return true;
		} catch (NoAlertPresentException e) {
			return false;
		}
	}

	private String closeAlertAndGetItsText() {
		try {
			Alert alert = driver.switchTo().alert();
			String alertText = alert.getText();
			if (acceptNextAlert) {
				alert.accept();
			} else {
				alert.dismiss();
			}
			return alertText;
		} finally {
			acceptNextAlert = true;
		}
	}

	// nouvelles Methodes pour les logs plus lisibles

	public void clickElement(By by) throws Exception {
		logger.info("Clic sur l'element '" + getPathFromBy(by) + "'");
		assertElementPresent(by);
		driver.findElement(by).click();
	}

	protected String getPathFromBy(By by){
		return by.toString().split(": ")[1]; // On a choisi de faire ce split pour gagner en concision, mais vous aurez peut-etre envie de conserver le type de selecteur.
	}

	public void assertElementPresent(By by) throws Exception {
		if(!isElementPresent(by)){
			logger.error("ERREUR : '" + getPathFromBy(by) + "' n'a pas ete trouve(e) sur la page.");
		}
		assertTrue(isElementPresent(by));
	}

	public boolean isElementPresent(By by) throws InterruptedException {
		boolean isElementPresent = false;
		int time = 0;
		while (!isElementPresent && time < timeOut) {
			if (isElementPresentNow(by)) {
				isElementPresent = true;
			} else {
				isElementPresent = false;
			}
			wait(5);
			time++;
		}
		return isElementPresent;
	}

	public boolean isElementPresentNow(By by) {
		driver.manage().timeouts().implicitlyWait(0, TimeUnit.MILLISECONDS);
		boolean isElementPresent = (driver.findElements(by).size() != 0);
		driver.manage().timeouts().implicitlyWait(timeOut, TimeUnit.MILLISECONDS);
		return isElementPresent;
	}
	
	// -- remlacement avec un selecteur
	protected void clickElement(Selecteur selecteur) throws Exception {
	    logger.info("Clic sur " + selecteur.getNom());
	    assertElementPresent(selecteur);
	    driver.findElement(selecteur.getChemin()).click();
	}
	protected String getPathFromBy(Selecteur selecteur){
	    return selecteur.getChemin().toString().split(": ")[1];
	}
	public void assertElementPresent(Selecteur selecteur) throws Exception {
	    if(!isElementPresent(selecteur.getChemin())){
	        logger.error("ERREUR : " + selecteur.getNom() +
	                " n'a pas ete trouve(e) sur la page. " +
	                " (selecteur : '" + getPathFromBy(selecteur) + "')");
	    }
	    assertTrue(isElementPresent(selecteur.getChemin()));
	}
	
	



	// --Methodes fonctionnelles

	// Methode pour aller sur une URL un nom unique
	
	

	// Methode pour creer un nom unique
	public static String YourUniqueName(){
		String generated_name = RandomStringUtils.randomAlphabetic(9);
		return generated_name;
	}
	// Methode pour creer un mail
	public static String YourUniqueMail(){
		String generated_mail = YourUniqueName() + "@macif.fr";
		return generated_mail;
	}

	// attente chargement de la page WEB
	public static void waitForLoad(WebDriver driver) {
		ExpectedCondition<Boolean> pageLoadCondition = new
				ExpectedCondition<Boolean>() {
			public Boolean apply(WebDriver driver) {
				return ((JavascriptExecutor)driver).executeScript("return document.readyState").equals("complete");
			}
		};
		WebDriverWait wait = new WebDriverWait(driver, 30);
		wait.until(pageLoadCondition);
	}


}



